#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['pygindex']

package_data = \
{'': ['*']}

package_dir = \
{'': 'src'}

setup(name='pygindex',
      version='0.0.1',
      description='PyGIndex - Python library/framework',
      author='Rytis Sileika',
      author_email='rts.pub@pulegium.eu',
      url='https://github.com/rytis/pygindex',
      packages=packages,
      package_data=package_data,
      package_dir=package_dir,
     )
